# -*- coding: utf-8 -*-

import sys
from twisted.python import log
from twisted.internet import reactor
from twisted.web.server import Site
from twisted.web.wsgi import WSGIResource
from .decorators import defer_to_thread
from .resource import WSGIRootResource
from observable import Observable

class Twisted(Observable):

    def __init__(self, app=None):
        self.app = None
        self.resources = {}

        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.app = app
        app.run = self.run

    def add_resource(self, name, resource):
        self.resources[name] = resource

    def create_site(self, resource, **options):
        return Site(resource)

    def run_simple(self, app, host, port, **options):
        self.trigger('run', app)

        if app.debug:
            log.startLogging(sys.stdout)

        resource = WSGIResource(reactor, reactor.getThreadPool(), app)
        site = self.create_site(WSGIRootResource(resource, self.resources), **options)

        reactor.listenTCP(int(port), site, interface=host)

        if 'run_reactor' in options and options['run_reactor'] is False:
            pass
        else:
            reactor.run()

    def run(self, host=None, port=None, debug=None, **options):
        app = self.app
        if app is None and 'app' in options:
            app = options['app']
        if app is None:
            raise Exception('"app" not defined')

        if host is None:
            host = '127.0.0.1'

        if port is None:
            server_name = app.config['SERVER_NAME']
            if server_name and ':' in server_name:
                port = int(server_name.rsplit(':', 1)[1])
            else:
                port = 5000

        if debug is not None:
            app.debug = bool(debug)

        self.run_simple(app, host, port, **options)



# -*- coding: utf-8 -*-

from twisted.web.resource import Resource

class WSGIRootResource(Resource):
    def __init__(self, wsgiResource, children):
        """
        Creates a Twisted Web root resource.
        """
        Resource.__init__(self)
        self._wsgiResource = wsgiResource
        self.children = children

    def getChild(self, path, request):
        request.prepath.pop()
        request.postpath.insert(0, path)
        return self._wsgiResource



# -*- coding: utf-8 -*-
"""
Flask-Twisted
-------------

Simple integration of Flask and Twisted
"""
from setuptools import setup

setup(
    name='Flask-Twisted',
    version='0.1.1',
    url='http://github.com/cravler/flask-twisted/',
    license='MIT',
    author='Sergei Vizel',
    author_email='sergei.vizel@gmail.com',
    description='Simple integration of Flask and Twisted',
    long_description=__doc__,
    py_modules=['flask_twisted'],
    packages=['flask_twisted'],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=[
        'Flask', 'Twisted', 'observable'
    ],
    test_suite='tests',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)


# -*- coding: utf-8 -*-

import functools
from twisted.internet import threads

def defer_to_thread(f):
    @functools.wraps(f)
    def wrapped(*args, **kwargs):
        return threads.deferToThread(f, *args, **kwargs)
    return wrapped
